import sys, struct, hashlib
from cassette import Wave, fsk, uart_sync, uart_sincronizer, get_fsk_blocks, get_manchester_blocks, irg_manchester, irg_fsk, zero_crossing, sum8

def dump_block(block):
  h = hashlib.md5()
  h.update(bytearray(block['data']))
  sys.stdout.write("%s %d %r\n" % (h.hexdigest(), len(block['data']), block['checksum']))

def clean_xex(xex):
  i=0
  while True:
    if xex[i+0]==0xff and xex[i+1]==0xff:
      i+=2
    elif i==0: return None
    start,end=xex[i+1]*256+xex[i+0],xex[i+3]*256+xex[i+2]
    if start>end or (i+end-start+5)>len(xex): return xex[:i]
    i+=4
    sys.stdout.write("segment: %04x-%04x\n" %  (start, end))
    i+=end-start+1


def main():
  sys.stdout.write("fsk2xex simple extractor 0.1\n")

  if len(sys.argv)<5:
    sys.stderr.write("usage: %s <input.wav> <output.xex> <block_start> <skip_bytes>\n" % sys.argv[0])
    return

  block_start=int(sys.argv[3])
  skip_bytes=int(sys.argv[4])

  w=Wave(sys.argv[1])

  fsk_blocks=get_fsk_blocks(w, min_irg=10000)
  xex=[]
  for f in fsk_blocks[block_start:]:
    dump_block(f)
    xex+=f['data'][skip_bytes:-1]
  
  xex=clean_xex(xex)
  if xex==None:
    sys.stderr.write("Invalid executable.\n")
  else:
    sys.stdout.write("Saving executable %s\n" % sys.argv[2])
    f=open(sys.argv[2], "wb")
    f.write(struct.pack("%dB" % len(xex), *xex))
    f.close()

main()
